@@ -26,6 +26,7 @@ import threading
 import time
 import uuid
 
+from amqp import exceptions as amqp_exec
 import kombu
 import kombu.connection
 import kombu.entity
@@ -59,7 +60,7 @@ else:
     # we will facing an issue by trying to override the threading module.
     stdlib_threading = threading
 
-# NOTE(sileht): don't exists in py2 socket module
+# NOTE(sileht): don't exist in py2 socket module
 TCP_USER_TIMEOUT = 18
 
 rabbit_opts = [
@@ -168,12 +169,23 @@ rabbit_opts = [
                default=2,
                help='How often times during the heartbeat_timeout_threshold '
                'we check the heartbeat.'),
-    cfg.IntOpt('direct_mandatory_flag',
-               default=True,
-               help='Enable/Disable the RabbitMQ mandatory flag '
-               'for direct send. The direct send is used as reply, '
-               'so the MessageUndeliverable exception is raised '
-               'in case the client queue does not exist.'),
+    cfg.BoolOpt('direct_mandatory_flag',
+                default=True,
+                deprecated_for_removal=True,
+                deprecated_reason='Mandatory flag no longer deactivable.',
+                help='(DEPRECATED) Enable/Disable the RabbitMQ mandatory '
+                'flag for direct send. The direct send is used as reply, '
+                'so the MessageUndeliverable exception is raised '
+                'in case the client queue does not exist.'
+                'MessageUndeliverable exception will be used to loop for a '
+                'timeout to lets a chance to sender to recover.'
+                'This flag is deprecated and it will not be possible to '
+                'deactivate this functionality anymore'),
+    cfg.BoolOpt('enable_cancel_on_failover',
+                default=False,
+                help="Enable x-cancel-on-ha-failover flag so that "
+                     "rabbitmq server will cancel and notify consumers"
+                     "when queue is down")
 ]
 
 LOG = logging.getLogger(__name__)
@@ -235,7 +247,8 @@ class Consumer(object):
 
     def __init__(self, exchange_name, queue_name, routing_key, type, durable,
                  exchange_auto_delete, queue_auto_delete, callback,
-                 nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0):
+                 nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0,
+                 enable_cancel_on_failover=False):
         """Init the Consumer class with the exchange_name, routing_key,
         type, durable auto_delete
         """
@@ -257,10 +270,16 @@ class Consumer(object):
             type=type,
             durable=self.durable,
             auto_delete=self.exchange_auto_delete)
+        self.enable_cancel_on_failover = enable_cancel_on_failover
 
     def declare(self, conn):
         """Re-declare the queue after a rabbit (re)connect."""
 
+        consumer_arguments = None
+        if self.enable_cancel_on_failover:
+            consumer_arguments = {
+                "x-cancel-on-ha-failover": True}
+
         self.queue = kombu.entity.Queue(
             name=self.queue_name,
             channel=conn.channel,
@@ -268,7 +287,9 @@ class Consumer(object):
             durable=self.durable,
             auto_delete=self.queue_auto_delete,
             routing_key=self.routing_key,
-            queue_arguments=self.queue_arguments)
+            queue_arguments=self.queue_arguments,
+            consumer_arguments=consumer_arguments
+        )
 
         try:
             LOG.debug('[%s] Queue.declare: %s',
@@ -469,6 +490,7 @@ class Connection(object):
         self.kombu_failover_strategy = driver_conf.kombu_failover_strategy
         self.kombu_compression = driver_conf.kombu_compression
         self.heartbeat_in_pthread = driver_conf.heartbeat_in_pthread
+        self.enable_cancel_on_failover = driver_conf.enable_cancel_on_failover
 
         if self.heartbeat_in_pthread:
             # NOTE(hberaud): Experimental: threading module is in use to run
@@ -496,6 +518,7 @@ class Connection(object):
             # if it was already monkey patched by eventlet/greenlet.
             global threading
             threading = stdlib_threading
+
         self.direct_mandatory_flag = driver_conf.direct_mandatory_flag
 
         if self.ssl:
@@ -1012,7 +1035,8 @@ class Connection(object):
                     except (socket.timeout,
                             ConnectRefuseError,
                             OSError,
-                            kombu.exceptions.OperationalError) as exc:
+                            kombu.exceptions.OperationalError,
+                            amqp_exec.ConnectionForced) as exc:
                         LOG.info("A recoverable connection/channel error "
                                  "occurred, trying to reconnect: %s", exc)
                         self.ensure_connection()
@@ -1118,31 +1142,35 @@ class Connection(object):
         responses for call/multicall
         """
 
-        consumer = Consumer(exchange_name='',  # using default exchange
-                            queue_name=topic,
-                            routing_key='',
-                            type='direct',
-                            durable=False,
-                            exchange_auto_delete=False,
-                            queue_auto_delete=False,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues,
-                            rabbit_queue_ttl=self.rabbit_transient_queues_ttl)
+        consumer = Consumer(
+            exchange_name='',  # using default exchange
+            queue_name=topic,
+            routing_key='',
+            type='direct',
+            durable=False,
+            exchange_auto_delete=False,
+            queue_auto_delete=False,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            rabbit_queue_ttl=self.rabbit_transient_queues_ttl,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
     def declare_topic_consumer(self, exchange_name, topic, callback=None,
                                queue_name=None):
         """Create a 'topic' consumer."""
-        consumer = Consumer(exchange_name=exchange_name,
-                            queue_name=queue_name or topic,
-                            routing_key=topic,
-                            type='topic',
-                            durable=self.amqp_durable_queues,
-                            exchange_auto_delete=self.amqp_auto_delete,
-                            queue_auto_delete=self.amqp_auto_delete,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues)
+        consumer = Consumer(
+            exchange_name=exchange_name,
+            queue_name=queue_name or topic,
+            routing_key=topic,
+            type='topic',
+            durable=self.amqp_durable_queues,
+            exchange_auto_delete=self.amqp_auto_delete,
+            queue_auto_delete=self.amqp_auto_delete,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
@@ -1153,16 +1181,18 @@ class Connection(object):
         exchange_name = '%s_fanout' % topic
         queue_name = '%s_fanout_%s' % (topic, unique)
 
-        consumer = Consumer(exchange_name=exchange_name,
-                            queue_name=queue_name,
-                            routing_key=topic,
-                            type='fanout',
-                            durable=False,
-                            exchange_auto_delete=True,
-                            queue_auto_delete=False,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues,
-                            rabbit_queue_ttl=self.rabbit_transient_queues_ttl)
+        consumer = Consumer(
+            exchange_name=exchange_name,
+            queue_name=queue_name,
+            routing_key=topic,
+            type='fanout',
+            durable=False,
+            exchange_auto_delete=True,
+            queue_auto_delete=False,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            rabbit_queue_ttl=self.rabbit_transient_queues_ttl,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
@@ -1286,7 +1316,7 @@ class Connection(object):
                 # the 404 kombu ChannelError and retry until the exchange
                 # appears
                 raise rpc_amqp.AMQPDestinationNotFound(
-                    "exchange %s doesn't exists" % exchange.name)
+                    "exchange %s doesn't exist" % exchange.name)
             raise
 
     def direct_send(self, msg_id, msg):
@@ -1368,6 +1398,10 @@ class RabbitDriver(amqpdriver.AMQPDriverBase):
         # the pool configuration properties
         max_size = conf.oslo_messaging_rabbit.rpc_conn_pool_size
         min_size = conf.oslo_messaging_rabbit.conn_pool_min_size
+        if max_size < min_size:
+            raise RuntimeError(
+                f"rpc_conn_pool_size: {max_size} must be greater than "
+                f"or equal to conn_pool_min_size: {min_size}")
         ttl = conf.oslo_messaging_rabbit.conn_pool_ttl
 
         connection_pool = pool.ConnectionPool(
