@@ -24,6 +24,8 @@ import threading
 
 import six
 
+from oslo_utils import eventletutils
+
 from oslo_messaging import _utils as utils
 from oslo_messaging import dispatcher
 from oslo_messaging import serializer as msg_serializer
@@ -218,7 +220,16 @@ class RPCDispatcher(dispatcher.DispatcherBase):
                 '(interval=%(interval)i)' % (
                     {'method': incoming.message.get('method'),
                      'interval': cm_heartbeat_interval}))
-            incoming.heartbeat()
+            try:
+                incoming.heartbeat()
+            except Exception as exc:
+                # The heartbeat message failed to send.  Likely the broker or
+                # client has died. Nothing to do here but exit the watchdog
+                # thread. If the client is still alive (dead broker) then its
+                # RPC will timeout as expected.
+                LOG.debug("Call-monitor heartbeat failed: %(exc)s"
+                          % ({'exc': exc}))
+                break
 
     def dispatch(self, incoming):
         """Dispatch an RPC message to the appropriate endpoint method.
@@ -240,7 +251,7 @@ class RPCDispatcher(dispatcher.DispatcherBase):
         # is executing if it runs for some time. The thread will wait
         # for the event to be signaled, which we do explicitly below
         # after dispatching the method call.
-        completion_event = threading.Event()
+        completion_event = eventletutils.Event()
         watchdog_thread = threading.Thread(target=self._watchdog,
                                            args=(completion_event, incoming))
         if incoming.client_timeout:
