@@ -21,6 +21,7 @@ import uuid
 
 import fixtures
 import kombu
+import kombu.connection
 import kombu.transport.memory
 from oslo_serialization import jsonutils
 from oslo_utils import eventletutils
@@ -32,6 +33,7 @@ from oslo_messaging._drivers import common as driver_common
 from oslo_messaging._drivers import impl_rabbit as rabbit_driver
 from oslo_messaging.exceptions import MessageDeliveryFailure
 from oslo_messaging.tests import utils as test_utils
+from oslo_messaging.transport import DriverLoadFailure
 from six.moves import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
@@ -142,6 +144,20 @@ class TestRabbitDriverLoad(test_utils.BaseTestCase):
 
         self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
 
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
+    def test_driver_load_max_less_than_min(self, fake_ensure, fake_reset):
+        self.config(
+            rpc_conn_pool_size=1, conn_pool_min_size=2,
+            group='oslo_messaging_rabbit')
+        self.messaging_conf.transport_url = self.transport_url
+        error = self.assertRaises(
+            DriverLoadFailure, oslo_messaging.get_transport, self.conf)
+        self.assertIn(
+            "rpc_conn_pool_size: 1 must be greater than or equal "
+            "to conn_pool_min_size: 2", str(error))
+
 
 class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
     scenarios = [
@@ -972,13 +988,19 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
             'kombu.connection.Connection.connection'))
         self.useFixture(fixtures.MockPatch(
             'kombu.connection.Connection.channel'))
+        # TODO(stephenfin): Drop hasattr when we drop support for kombo < 4.6.8
+        if hasattr(kombu.connection.Connection, '_connection_factory'):
+            self.useFixture(fixtures.MockPatch(
+                'kombu.connection.Connection._connection_factory'))
 
         # starting from the first broker in the list
         url = oslo_messaging.TransportURL.parse(self.conf, None)
         self.connection = rabbit_driver.Connection(self.conf, url,
                                                    driver_common.PURPOSE_SEND)
-        self.useFixture(fixtures.MockPatch(
-            'kombu.connection.Connection.connect'))
+        # TODO(stephenfin): Remove when we drop support for kombo < 4.6.8
+        if hasattr(kombu.connection.Connection, 'connect'):
+            self.useFixture(fixtures.MockPatch(
+                'kombu.connection.Connection.connect'))
         self.addCleanup(self.connection.close)
 
     def test_ensure_four_retry(self):
