@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import fixtures
 import threading
 
 from oslo_config import cfg
@@ -22,7 +23,6 @@ import oslo_messaging
 from oslo_messaging.notify import dispatcher
 from oslo_messaging.notify import notifier as msg_notifier
 from oslo_messaging.tests import utils as test_utils
-import six
 from six.moves import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
@@ -131,28 +131,25 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
     def setUp(self):
         super(TestNotifyListener, self).setUp(conf=cfg.ConfigOpts())
         ListenerSetupMixin.setUp(self)
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
-    @mock.patch('debtcollector.deprecate')
-    def test_constructor(self, deprecate):
+    def test_constructor(self):
         transport = msg_notifier.get_notification_transport(
             self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo')
         endpoints = [object()]
 
         listener = oslo_messaging.get_notification_listener(
-            transport, [target], endpoints)
+            transport, [target], endpoints, executor='threading')
 
         self.assertIs(listener.conf, self.conf)
         self.assertIs(listener.transport, transport)
         self.assertIsInstance(listener.dispatcher,
                               dispatcher.NotificationDispatcher)
         self.assertIs(listener.dispatcher.endpoints, endpoints)
-        self.assertEqual('blocking', listener.executor_type)
-        deprecate.assert_called_once_with(
-            'blocking executor is deprecated. Executor default will be '
-            'removed. Use explicitly threading or eventlet instead',
-            removal_version='rocky', version='pike',
-            category=FutureWarning)
+        self.assertEqual('threading', listener.executor_type)
 
     def test_no_target_topic(self):
         transport = msg_notifier.get_notification_transport(
@@ -183,7 +180,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
             self.assertTrue(False)
 
     def test_batch_timeout(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.return_value = None
@@ -191,7 +189,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, 1))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(12):
+        for _ in range(12):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(3)
@@ -209,7 +207,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                         mock.call(messages * 2)])
 
     def test_batch_size(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.return_value = None
@@ -217,7 +216,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, None))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(10):
+        for _ in range(10):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(2)
@@ -234,7 +233,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                         mock.call(messages * 5)])
 
     def test_batch_size_exception_path(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.side_effect = [None, Exception('boom!')]
@@ -242,7 +242,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, None))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(10):
+        for _ in range(10):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(2)
@@ -506,3 +506,18 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
         for call in mocked_endpoint1_calls:
             self.assertIn(call, endpoint2.info.mock_calls +
                           endpoint3.info.mock_calls)
+
+
+class TestListenerTransportWarning(test_utils.BaseTestCase):
+
+    @mock.patch('oslo_messaging.notify.listener.LOG')
+    def test_warning_when_rpc_transport(self, log):
+        transport = oslo_messaging.get_rpc_transport(self.conf)
+        target = oslo_messaging.Target(topic='foo')
+        endpoints = [object()]
+        oslo_messaging.get_notification_listener(
+            transport, [target], endpoints)
+        log.warning.assert_called_once_with(
+            "Using RPC transport for notifications. Please use "
+            "get_notification_transport to obtain a "
+            "notification transport instance.")
