@@ -42,21 +42,53 @@ class TestCastCall(test_utils.BaseTestCase):
 
     def test_cast_call(self):
         self.config(rpc_response_timeout=None)
-
+        transport_options = oslo_messaging.TransportOptions()
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target())
+        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
+                                          transport_options=transport_options)
 
         transport._send = mock.Mock()
 
         msg = dict(method='foo', args=self.args)
-        kwargs = {'retry': None}
+        kwargs = {'retry': None, 'transport_options': transport_options}
         if self.call:
             kwargs['wait_for_reply'] = True
             kwargs['timeout'] = None
+            kwargs['call_monitor_timeout'] = None
+
+        method = client.call if self.call else client.cast
+        method(self.ctxt, 'foo', **self.args)
+        self.assertFalse(transport_options.at_least_once)
+        transport._send.assert_called_once_with(oslo_messaging.Target(),
+                                                self.ctxt,
+                                                msg,
+                                                **kwargs)
+
+    def test_cast_call_with_transport_options(self):
+        self.config(rpc_response_timeout=None)
+
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+
+        transport_options = oslo_messaging.TransportOptions(at_least_once=True)
+        client = oslo_messaging.RPCClient(
+            transport,
+            oslo_messaging.Target(),
+            transport_options=transport_options)
+
+        transport._send = mock.Mock()
+
+        msg = dict(method='foo', args=self.args)
+        kwargs = {'retry': None,
+                  'transport_options': transport_options}
+        if self.call:
+            kwargs['wait_for_reply'] = True
+            kwargs['timeout'] = None
+            kwargs['call_monitor_timeout'] = None
 
         method = client.call if self.call else client.cast
         method(self.ctxt, 'foo', **self.args)
 
+        self.assertTrue(transport_options.at_least_once)
         transport._send.assert_called_once_with(oslo_messaging.Target(),
                                                 self.ctxt,
                                                 msg,
@@ -202,7 +234,8 @@ class TestCastToTarget(test_utils.BaseTestCase):
         transport._send.assert_called_once_with(expect_target,
                                                 {},
                                                 msg,
-                                                retry=None)
+                                                retry=None,
+                                                transport_options=None)
 
 
 TestCastToTarget.generate_scenarios()
@@ -215,19 +248,21 @@ class TestCallTimeout(test_utils.BaseTestCase):
 
     scenarios = [
         ('all_none',
-         dict(confval=None, ctor=None, prepare=_notset, expect=None)),
+         dict(confval=None, ctor=None, prepare=_notset, expect=None, cm=None)),
         ('confval',
-         dict(confval=21, ctor=None, prepare=_notset, expect=21)),
+         dict(confval=21, ctor=None, prepare=_notset, expect=21, cm=None)),
         ('ctor',
-         dict(confval=None, ctor=21.1, prepare=_notset, expect=21.1)),
+         dict(confval=None, ctor=21.1, prepare=_notset, expect=21.1, cm=None)),
         ('ctor_zero',
-         dict(confval=None, ctor=0, prepare=_notset, expect=0)),
+         dict(confval=None, ctor=0, prepare=_notset, expect=0, cm=None)),
         ('prepare',
-         dict(confval=None, ctor=None, prepare=21.1, expect=21.1)),
+         dict(confval=None, ctor=None, prepare=21.1, expect=21.1, cm=None)),
         ('prepare_override',
-         dict(confval=None, ctor=10.1, prepare=21.1, expect=21.1)),
+         dict(confval=None, ctor=10.1, prepare=21.1, expect=21.1, cm=None)),
         ('prepare_zero',
-         dict(confval=None, ctor=None, prepare=0, expect=0)),
+         dict(confval=None, ctor=None, prepare=0, expect=0, cm=None)),
+        ('call_monitor',
+         dict(confval=None, ctor=None, prepare=60, expect=60, cm=30)),
     ]
 
     def test_call_timeout(self):
@@ -235,12 +270,14 @@ class TestCallTimeout(test_utils.BaseTestCase):
 
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          timeout=self.ctor)
+                                          timeout=self.ctor,
+                                          call_monitor_timeout=self.cm)
 
         transport._send = mock.Mock()
 
         msg = dict(method='foo', args={})
-        kwargs = dict(wait_for_reply=True, timeout=self.expect, retry=None)
+        kwargs = dict(wait_for_reply=True, timeout=self.expect, retry=None,
+                      call_monitor_timeout=self.cm, transport_options=None)
 
         if self.prepare is not _notset:
             client = client.prepare(timeout=self.prepare)
@@ -272,7 +309,8 @@ class TestCallRetry(test_utils.BaseTestCase):
 
         msg = dict(method='foo', args={})
         kwargs = dict(wait_for_reply=True, timeout=60,
-                      retry=self.expect)
+                      retry=self.expect, call_monitor_timeout=None,
+                      transport_options=None)
 
         if self.prepare is not _notset:
             client = client.prepare(retry=self.prepare)
@@ -329,9 +367,11 @@ class TestSerializer(test_utils.BaseTestCase):
                                           serializer=serializer)
 
         transport._send = mock.Mock()
-
-        kwargs = dict(wait_for_reply=True, timeout=None) if self.call else {}
+        kwargs = dict(wait_for_reply=True,
+                      timeout=None) if self.call else {}
         kwargs['retry'] = None
+        if self.call:
+            kwargs['call_monitor_timeout'] = None
 
         transport._send.return_value = self.retval
 
@@ -360,6 +400,7 @@ class TestSerializer(test_utils.BaseTestCase):
         transport._send.assert_called_once_with(oslo_messaging.Target(),
                                                 dict(user='alice'),
                                                 msg,
+                                                transport_options=None,
                                                 **kwargs)
         expected_calls = [mock.call(self.ctxt, arg) for arg in self.args]
         self.assertEqual(expected_calls,
@@ -441,6 +482,7 @@ class TestVersionCap(test_utils.BaseTestCase):
             if self.call:
                 kwargs['wait_for_reply'] = True
                 kwargs['timeout'] = None
+                kwargs['call_monitor_timeout'] = None
 
         prep_kwargs = {}
         if self.prepare_cap is not _notset:
@@ -458,7 +500,10 @@ class TestVersionCap(test_utils.BaseTestCase):
             self.assertFalse(self.success)
         else:
             self.assertTrue(self.success)
-            transport._send.assert_called_once_with(target, {}, msg, **kwargs)
+            transport._send.assert_called_once_with(target, {}, msg,
+                                                    transport_options=None,
+                                                    **kwargs)
+
 
 TestVersionCap.generate_scenarios()
 
